package syncloud.synchronization;

import org.syncloud.facade.StorageManager;
import org.syncloud.facade.dao.AccountsDao;
import org.syncloud.facade.dao.SyncDao;
import syncloud.core.log.Logger;
import syncloud.storage.*;

import java.util.ArrayList;
import java.util.List;
import java.util.TimerTask;

public class Synchronization extends TimerTask {

    private static Logger logger = Logger.getLogger(Synchronization.class);

    private SyncDao syncDao;
    private StorageManager storageManager;
    private ISyncRecord states;
    private AccountsDao accountsDao;

    public Synchronization(
            SyncDao syncDao,
            StorageManager storageManager,
            ISyncRecord states, AccountsDao accountsDao) {
        super();
        this.syncDao = syncDao;
        this.storageManager = storageManager;
        this.states = states;
        this.accountsDao = accountsDao;
    }

    @Override
    public void run() {

        logger.info("synchronization cycle started");

        List<Sync> syncs = syncDao.getSyncs();
        List<IFolder> folders = new ArrayList<IFolder>();
        for (Sync sync : syncs) {
            logger.debug("found sync: " + sync.getName());
            for (NodeKey nodeKey : sync.getPaths()) {
                StorageKey storageKey = nodeKey.getStorageKey();
                IStorage storage = storageManager.getStorage(storageKey);
                if (storage instanceof ISecured) {
                    Account account = accountsDao.getUserAccount(storageKey);
                    if (account == null) {
                        logger.info("no saved account info for the sync, skipping");
                        continue;
                    }
                    String login = account.getUser().getLogin();
                    boolean authenticated = ((ISecured) storage).authenticate(account.getPassword());
                    if (!authenticated){
                        logger.error(String.format("unable to authenticate %s, will continue with other sync paths ...", login));
                        continue;
                    }

                }
                logger.debug("adding path to synchronize: " + nodeKey.getNativePath());
                try {
                    folders.add(new StorageUtils(storage).getOrCreateFolder(nodeKey.getPathKey()));
                } catch (StorageException e) {
                    logger.error("unable to get/create sync folder, error: " + e.getMessage());
                }
            }
        }

        Synchronizer synchronizer = new Synchronizer(states, null);
        synchronizer.synchronize(folders);

        logger.info("synchronization cycle finished");
    }
}
