package syncloud.synchronization;

import syncloud.core.CoreException;
import syncloud.core.log.Logger;
import syncloud.synchronization.nodes.FoldersSyncBundle;
import syncloud.synchronization.nodes.SyncNode;
import syncloud.storage.IFolder;

import java.util.ArrayList;
import java.util.List;

public class Synchronizer implements ISynchronizer {

    private static Logger logger = Logger.getLogger(Synchronizer.class);


    private ISyncRecord record;
    private IConflictResolver resolver;

    public Synchronizer(ISyncRecord record, IConflictResolver resolver) {
        this.record = record;
        this.resolver = resolver;
    }

    public ISyncRecord getRecord() {
        return this.record;
    }

    public IConflictResolver getResolver() {
        return this.resolver;
    }

    public void synchronize(List<IFolder> folders) throws CoreException {
        logger.debug("folders to synchronize: " + folders.size());
        List<SyncNode> syncNodes = new ArrayList<SyncNode>();
        for (IFolder folder: folders) {
            SyncNode syncNode = new SyncNode(folder.getKey().getNativePath(), null, folder);
            syncNodes.add(syncNode);
        }
        FoldersSyncBundle syncBundle = new FoldersSyncBundle(this, syncNodes);
        syncBundle.synchronizeChildren();
    }
}
